﻿//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//      The Cy4NET System                      Version 19.10                            //
//      __________________________             _________________________________        //
//                                                                                      //
//      Decentralised Realtime                 Copyright (c) 2012 Reimar Grasbon        //
//      Peer-to-peer Communication             (www.cy4net.org)                         //
//      Protocol                                                                        //
//      _________________________              _________________________________        //
//                                                                                      //
//      [Service Unit]                                                                  //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  This software is subjected to the FreeBSD License. Redistribution and use in source //
//  and binary forms, with or without modification, are permitted provided that the     //
//  following conditions are met:                                                       //
//                                                                                      //
//  1. Redistributions of source code must retain the above copyright notice, this list //
//     of conditions and the following disclaimer.                                      //
//                                                                                      //
//  2. Redistributions in binary form must reproduce the above copyright notice, this   //
//     list of conditions and the following disclaimer in the documentation and/or      //
//     other materials provided with the distribution.                                  //
//                                                                                      //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY //
//  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED           //
//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  //
//  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,    //
//  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT  //
//  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR  //
//  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,   //
//  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)  //
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE          //
//  POSSIBILITY OF SUCH DAMAGE.                                                         //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////

#include "Cy4Service.h"

//////////////////////////////////////////////////////////////////////////////////////////
//  Funktionen  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
UInt32
CY4SERVICE_Base(const Cy4Service* service)
{
    return service->base;
}

//////////////////////////////////////////////////////////////////////////////////////////
UInt16
CY4SERVICE_Size(const Cy4Service* service)
{
    return service->size;
}

//////////////////////////////////////////////////////////////////////////////////////////
Cy4ServiceAccess
CY4SERVICE_Access(const Cy4Service* service)
{
    return service->access;
}

//////////////////////////////////////////////////////////////////////////////////////////
bool
CY4SERVICE_IsValid(const Cy4Service* service)
{
    return service->size != 0;
}

//////////////////////////////////////////////////////////////////////////////////////////
//  CY4SERVICE_TransferPayload(): Komfortfunktion. Je nach Zugriffsart der Nachricht    //
//  ----------------------------  bewegt diese Funktion Nutzlast aus der Nachricht ins  //
//  Datenfeld des Dienstes oder umgekehrt.                                              //
//                                                                                      //
//  Die aufrufende Umgebung hat dabei sicherzustellen, dass <data> auf ein, dem Dienst  //
//  entsprechend, ausreichend dimensioniertes Bytefeld zeigt. CY4SERVICE_TransferPay-   //
//  load() kommt üblicherweise in den Dienstbearbeitern zum Einsatz.                    //
//                                                                                      //
//  Eingabe: · service: Der zugrunde liegende Dienst.                                   //
//           · message: Je nach Zugriffsart der Nachricht: Bei CY4MESSAGE_WRITE wird    //
//                      Nutzlast aus der Nachricht ins Datenfeld des Dienstes über-     //
//                      tragen. Bei CY4MESSAGE_READ ist es umgekehrt, die Nachricht     //
//                      nimmt Nutzlast aus dem Datenfeld des Dienstes auf.              //
//           · data:    Datenfeld des Dienstes.                                         //
//  Ausgabe: -                                                                          //
//////////////////////////////////////////////////////////////////////////////////////////
void
CY4SERVICE_TransferPayload(const Cy4Service* service, Cy4Message* message, UInt08* data)
{
    UInt08 j = (UInt08)(message->location - service->base);
    switch (message->access)
    {   case CY4MESSAGE_READ:
            // Servicedaten -> Nutzlast
            for (UInt08 i=0; i<message->count; i++)
            {   message->data[i] = data[j];
                j++;
            }
            break;
        case CY4MESSAGE_WRITE:
            // Nutzlast -> Servicedaten
            for (UInt08 i=0; i<message->count; i++)
            {   data[j] = message->data[i];
                j++;
            }
            break;
        default:
            ;
    }
}

//////////////////////////////////////////////////////////////////////////////////////////
