﻿//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Betriebsmodi                                                                        //
//  _________________________________                                                   //
//                                                                                      //
//  Copyright (c) 2021 Reimar Grasbon                                                   //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  This software is subjected to the FreeBSD License. Redistribution and use in source //
//  and binary forms, with or without modification, are permitted provided that the     //
//  following conditions are met:                                                       //
//                                                                                      //
//  1. Redistributions of source code must retain the above copyright notice, this list //
//     of conditions and the following disclaimer.                                      //
//                                                                                      //
//  2. Redistributions in binary form must reproduce the above copyright notice, this   //
//     list of conditions and the following disclaimer in the documentation and/or      //
//     other materials provided with the distribution.                                  //
//                                                                                      //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY //
//  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED           //
//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  //
//  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,    //
//  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT  //
//  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR  //
//  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,   //
//  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)  //
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE          //
//  POSSIBILITY OF SUCH DAMAGE.                                                         //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////

#include "Main.h"
#include "Cy4Net.h"
#include "Storing.h"
#include "Board.h"
#include "Timer.h"
#include "String.h"

//////////////////////////////////////////////////////////////////////////////////////////
//  Typen  ///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
typedef enum
{   PINGTEST_Outstanding,
    PINGTEST_Busy,
    PINGTEST_Passed,
    PINGTEST_Failed    
} PingTest;

//////////////////////////////////////////////////////////////////////////////////////////
//  Daten  ///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
static UInt08   pingTestCounter = 0;
static PingTest pingTestState[8];

//////////////////////////////////////////////////////////////////////////////////////////
//  Lokale Funktionen   //////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
static void
ClearInputEvents(void)
{
	BOARD_TakePushButton(PUSHBUTTON_DIRECT);
	BOARD_TakePushButton(PUSHBUTTON_ENCODER);
	BOARD_TakeEncoder();
}

//////////////////////////////////////////////////////////////////////////////////////////
static void
Wait(UInt08 seconds)	 // 0 .. 10
{
	Timer    timer = TIMER_MILLISECONDS(seconds*1000);
    TIMER_Restart(&timer);
	while (!TIMER_HasExpired(&timer) && MAIN_DoIdleProcessing());
}

//////////////////////////////////////////////////////////////////////////////////////////
static void
PrintPingTestState(String* text)
{    
    for (UInt08 i=0; i<CARDINALITY(pingTestState); i++)
    {   char c;
        switch (pingTestState[i])
        {   case PINGTEST_Outstanding: c = '\x10'; break;
            case PINGTEST_Busy:        c = '\x11'; break;  
            case PINGTEST_Passed:      c = '\x12'; break;
            case PINGTEST_Failed:      c = '\x13'; break;
            default:                   c = '?';
        }
        STRING_PrintChar(text, c);
    }
}

//////////////////////////////////////////////////////////////////////////////////////////
static void
PrintXXX(String* text, UInt08 val)
{
	if( val < 100 ) STRING_PrintChar(text, '0');
	if( val <  10 ) STRING_PrintChar(text, '0');
	STRING_PrintNumber(text, (SInt16)val);
}

//////////////////////////////////////////////////////////////////////////////////////////
static void
ModeChangeAddress(Cy4Address* address)
{
	typedef enum
	{   NET,
		DEV
	} AddressComponent;

	ClearInputEvents();
	Cy4Address       newAddress        = *address;
	AddressComponent component         = NET;
	Timer            inactivityTimeout = TIMER_MILLISECONDS(CFG_MODE_INACTIVITY_TIMEOUT*1000);
	Timer            timer             = TIMER_MILLISECONDS(300);	
	bool             blink             = true;
	SInt08           oldPosition       = 127;
	bool             refresh           = true;
	TIMER_Restart(&inactivityTimeout);
	while (!TIMER_HasExpired(&inactivityTimeout) && MAIN_DoIdleProcessing())
	{   // Addresskomponente ändern
		SInt08 newPosition = BOARD_TakeEncoder();
		if( oldPosition != newPosition )
		{   oldPosition  = newPosition;
			UInt08 net = CY4ADDRESS_Net(&newAddress);
			UInt08 dev = CY4ADDRESS_Dev(&newAddress);			
			if( component == NET ) net += newPosition;
			else                   dev += newPosition;			
			// Rundrufe verbieten
			if (net == 0 || dev == 0)
			{	if (net == 0)
				{	if      (newPosition > 0) net =   1;
					else if (newPosition < 0) net = 255;
				}
				if (dev == 0)
				{	if      (newPosition > 0) dev =   1;
					else if (newPosition < 0) dev = 255;
				}
			}									
			// 255.255 verbieten
			if (net == 255 && dev == 255)
			{	if      (newPosition > 0) dev =   1;
				else if (newPosition < 0) dev = 254;
			}
			newAddress = CY4ADDRESS_Create(net, dev);			
			TIMER_Restart(&inactivityTimeout);
			refresh = true;
		}
		if( TIMER_RestartIfExpired(&timer) )
		{   blink = !blink;
			refresh = true;
		}
		// Addresskomponenten anzeigen
		if( refresh )
		{   String text = STRING_EMPTY;
			STRING_PrintChar(&text, '\r');
			if( component == NET )
			{   char c;
				if( blink ) c = '\176';
				else        c = ' ';
				STRING_PrintChar(&text, c);
				PrintXXX(&text, newAddress.net);
				if( blink ) c = '\177';
				else        c = '.';
				STRING_PrintChar(&text, c);
				PrintXXX(&text, newAddress.dev);
			}
			else // changeComp == DEV
			{   PrintXXX(&text, newAddress.net);
				char c;
				if( blink ) c = '\176';
				else        c = '.';
				STRING_PrintChar(&text, c);
				PrintXXX(&text, newAddress.dev);
				if( blink ) c = '\177';
				else        c = ' ';
				STRING_PrintChar(&text, c);
			}
			BOARD_PrintOnDisplay(STRING_ConstData(&text));
			refresh = false;
		}
		if (BOARD_TakePushButton(PUSHBUTTON_ENCODER))
		{   if( component == DEV ) break;
			component = DEV;
		}
	}
	if( TIMER_HasExpired(&inactivityTimeout) )
	{   if( component == NET ) BOARD_PrintOnDisplay("\r xxx.xxx");
		else                   BOARD_PrintOnDisplay("\rxxx.xxx ");
		Wait(1);
	}
	else
	{   BOARD_PrintOnDisplay("\r   Ok   ");
		Wait(1);
		*address = newAddress;
	}
	ClearInputEvents();
}

//////////////////////////////////////////////////////////////////////////////////////////
//  Cy4-Nachrichtenausstatter  ///////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
static void
SetupPingMessage(Cy4Message* message)
{
    Cy4Address address;
    STORING_Load(STO_DestinationAddress, &address);
    CY4MESSAGE_SetupRead(message, address, 0xFFFFF0, 7);  // CallTypePlate
    pingTestState[pingTestCounter] = PINGTEST_Busy;
	String text = STRING_EMPTY;
	STRING_PrintChar(&text, '\r');
    PrintPingTestState(&text);	
	BOARD_PrintOnDisplay(STRING_ConstData(&text));
}

//////////////////////////////////////////////////////////////////////////////////////////
//  Cy4-Dienst Antwortbehandler  /////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
static void
ResponseOfPingMessage(const Cy4Message* message)
{       
    if (CY4MESSAGE_IsValid(message)) pingTestState[pingTestCounter] = PINGTEST_Passed;
    else                             pingTestState[pingTestCounter] = PINGTEST_Failed;
    String text = STRING_EMPTY;
	STRING_PrintChar(&text, '\r');
    PrintPingTestState(&text);    
    BOARD_PrintOnDisplay(STRING_ConstData(&text));
    pingTestCounter++;    
}

//////////////////////////////////////////////////////////////////////////////////////////
//  Funktionen  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
void
MODE_ShowStartInfo(void)
{
    BOARD_PrintOnDisplay("\n Cy4-LT");
    Wait(CFG_SPLASH_SCREEN_DURATION);
    ClearInputEvents();
}

//////////////////////////////////////////////////////////////////////////////////////////
void
MODE_SendPingSequence(void)
{   	
    // Pingtest Anfragen abschicken
	pingTestCounter = 0;
    for (UInt08 i=0; i<sizeof(pingTestState); i++)
    {   pingTestState[i] = PINGTEST_Outstanding;
        CY4NET_Post(SetupPingMessage, ResponseOfPingMessage);
    }
	// Warten, bis alle Anfragen abgearbeitet sind
	while (pingTestCounter < sizeof(pingTestState))
	{	if (!MAIN_DoIdleProcessing()) break;
	}
	// Ergebnis für kurze Zeit anzeigen, danach Ping-Test beenden
	Wait(2);
	ClearInputEvents();	
}

//////////////////////////////////////////////////////////////////////////////////////////
void
MODE_SetOwnAddress(void)
{	
	Cy4Address address;
	BOARD_PrintOnDisplay("\rOwn Adr:");
	Wait(1);
	address = STORING_LoadCy4SelfAdr();
	ModeChangeAddress(&address);
	STORING_SaveCy4SelfAdr(address);
	CY4NET_SetAddress(address);	
}

//////////////////////////////////////////////////////////////////////////////////////////
void    
MODE_SetDestinationAddress(void)
{	
	Cy4Address address;
	BOARD_PrintOnDisplay("\rDest.Adr");
	Wait(1);
	STORING_Load(STO_DestinationAddress, &address);
	ModeChangeAddress(&address);
	STORING_Save(&address, STO_DestinationAddress);	
}

//////////////////////////////////////////////////////////////////////////////////////////

