﻿//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  Passiver Zeitgeber                                                                  //
//  _________________________________                                                   //
//                                                                                      //
//  Copyright (c) 2007 Reimar Grasbon                                                   //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  This software is subjected to the FreeBSD License. Redistribution and use in source //
//  and binary forms, with or without modification, are permitted provided that the     //
//  following conditions are met:                                                       //
//                                                                                      //
//  1. Redistributions of source code must retain the above copyright notice, this list //
//     of conditions and the following disclaimer.                                      //
//                                                                                      //
//  2. Redistributions in binary form must reproduce the above copyright notice, this   //
//     list of conditions and the following disclaimer in the documentation and/or      //
//     other materials provided with the distribution.                                  //
//                                                                                      //
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY //
//  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED           //
//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  //
//  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,    //
//  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT  //
//  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR  //
//  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,   //
//  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)  //
//  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE          //
//  POSSIBILITY OF SUCH DAMAGE.                                                         //
//                                                                                      //
//////////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "TypeDefs.h"

//////////////////////////////////////////////////////////////////////////////////////////
//  Definitionen  ////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
#define TIMER_MILLISECONDS(time) { .duration   = time,                   \
                                   .base       = TIMERBASE_MILLISECONDS, \
                                   .hasExpired = true,                   \
                                   .ticksStamp = 0                       \
                                 }
#define TIMER_SECONDS(time)      { .duration   = time,                   \
                                   .base       = TIMERBASE_SECONDS,      \
                                   .hasExpired = true,                   \
                                   .ticksStamp = 0                       \
                                 }

//////////////////////////////////////////////////////////////////////////////////////////
//  Typen  ///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
typedef enum                        // Zeitbasis:
{   TIMERBASE_MILLISECONDS = 0,     // · Millisekunden-Zeitgeber
    TIMERBASE_SECONDS      = 1      // · Sekunden-Zeitgeber
} TimerBase;

typedef struct                      // Zeitgeber:
{   UInt16      duration    : 14;   // Dauer: 0 .. 15000 [Ticks]
    TimerBase   base        :  1;   // Zeitbasis [TimerBase]
    bool        hasExpired  :  1;   // 'true', wenn Zeitgeber abgelaufen ist
    UInt16      ticksStamp;         // Letzter Zeitstempel [Ticks]
} Timer;

//////////////////////////////////////////////////////////////////////////////////////////
//  Funktionen  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
void      TIMER_Start(Timer*, UInt16);
void      TIMER_Restart(Timer*);
void      TIMER_Stop(Timer*);
bool      TIMER_HasExpired(Timer*);
bool      TIMER_RestartIfExpired(Timer*);
UInt16    TIMER_Elapsed(Timer*);
UInt16    TIMER_Remaining(Timer*);
UInt16    TIMER_Duration(const Timer*);
void      TIMER_SetBase(Timer*, TimerBase);
TimerBase TIMER_Base(const Timer*);

//////////////////////////////////////////////////////////////////////////////////////////

